// Fill out your copyright notice in the Description page of Project Settings.


#include "BountyHunterCharacter.h"
#include "Kismet/GameplayStatics.h"
#include "EnhancedInputComponent.h"

void ABountyHunterCharacter::SpawnMainBounty(UEVGameInstance* GI)
{
	CurrentMainBountyIndex = GI->MainSaveGameInstanceRef->LastMainBountyIndexInOpenWorld;

	if (GI->MainSaveGameInstanceRef->CurrentMainBountyIndexSave > CurrentMainBountyIndex) CompleteCurrentMainBounty(GI);
		
	if (MainBountyClasses.IsEmpty() || MainBountyClasses.Num() <= CurrentMainBountyIndex|| !IsValid(MainBountyClasses[CurrentMainBountyIndex]))
	{
		AllBountiesCompleted();
		if (CurrentMainBountyIndex >= 2 && !(UGameplayStatics::GetCurrentLevelName(GetWorld()) == "EnemyAITest"))
		{
			UGameplayStatics::OpenLevel(GetWorld(), FName("EnemyAITest"));
		}
		return;
	}

	CurrentMainBounty = GetWorld()->SpawnActor<AMainBountyClass>(MainBountyClasses[CurrentMainBountyIndex]);
	const FAttachmentTransformRules AttachmentTransformRules(EAttachmentRule::SnapToTarget, true);
	CurrentMainBounty->AttachToComponent(GetRootComponent(), AttachmentTransformRules);

	UGameplayStatics::GetCurrentLevelName(GetWorld()) == OpenWorldLevelName ? CurrentMainBounty->SpawnOpenWorldCheckpoint() : CurrentMainBounty->SpawnCheckpoints();
}

void ABountyHunterCharacter::CompleteCurrentMainBounty(UEVGameInstance* GI)
{
	// Collect rewards for bounty and move onto next
	UE_LOG(LogTemp, Warning, TEXT("Would be collecting reward for completing bounty %d"), CurrentMainBountyIndex);
	
	CurrentMainBountyIndex = GI->MainSaveGameInstanceRef->CurrentMainBountyIndexSave;
	GI->MainSaveGameInstanceRef->LastMainBountyIndexInOpenWorld = CurrentMainBountyIndex;
	UGameplayStatics::SaveGameToSlot(GI->MainSaveGameInstanceRef, "MainSave", 0);
}

void ABountyHunterCharacter::AllBountiesCompleted()
{
	UE_LOG(LogTemp, Warning, TEXT("Game Completed!!!! No more Bounties left... or you forgot to set a bounty at one of the indexes oof"));
}

void ABountyHunterCharacter::BeginPlay()
{
	UEVGameInstance* GI = Cast<UEVGameInstance>(GetGameInstance());
	if (IsValid(GI->MainSaveGameInstanceRef))
	{
		SpawnMainBounty(GI);
		FTransform SpawnLoc = GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave;
		if (SpawnLoc.GetLocation() != FVector(0, 0, 0) && UGameplayStatics::GetCurrentLevelName(GetWorld()) == OpenWorldLevelName)
		{
			SetActorTransform(GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave);
			Cast<APlayerController>(GetController())->SetControlRotation(GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave.GetRotation().Rotator());
		} 
	}
	Super::BeginPlay();
}

void ABountyHunterCharacter::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
	if (UEnhancedInputComponent* EnhancedInputComponent = CastChecked<UEnhancedInputComponent>(PlayerInputComponent))
	{
		EnhancedInputComponent->BindAction(PauseMenuAction, ETriggerEvent::Started, this, &ABountyHunterCharacter::TogglePauseMenu);
	}
	
	UE_LOG(LogTemp, Warning, TEXT("Setup player input on bounty hunter character"));

	Super::SetupPlayerInputComponent(PlayerInputComponent);
}

void ABountyHunterCharacter::TogglePauseMenu()
{
	bool bOpenNewPauseMenu = !IsValid(PauseMenu);
	FInputModeUIOnly UIOnly;
	FInputModeGameOnly GameOnly;
	APlayerController* PC = Cast<APlayerController>(GetController());

	InPauseMenu = bOpenNewPauseMenu;
	PC->SetIgnoreMoveInput(bOpenNewPauseMenu);
	PC->SetIgnoreLookInput(bOpenNewPauseMenu);
	PC->SetShowMouseCursor(bOpenNewPauseMenu);
	if (bOpenNewPauseMenu)
	{
		PauseMenu = CreateWidget<UPauseMenuClass>(GetWorld(), PauseMenuWidgetClass);
		PauseMenu->AddToViewport(50);
		BountiesTab = CreateWidget(GetWorld(), BountyTabWidgetClass);
		PauseMenu->OpenNewTab(BountiesTab);
	}
	else
	{
		PauseMenu->RemoveFromParent();
		PauseMenu = nullptr;
	}
}

void ABountyHunterCharacter::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
}

void ABountyHunterCharacter::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
	Super::EndPlay(EndPlayReason);
	if (UGameplayStatics::GetCurrentLevelName(GetWorld()) != OpenWorldLevelName) return;
	UEVGameInstance* GI = Cast<UEVGameInstance>(GetGameInstance());
	if (!IsValid(GI->MainSaveGameInstanceRef)) return;
	TArray<AActor*> LandingZones;
	ALandingZone* ClosestLandingZone = nullptr;
	double DistToClosestLandingZone;
	UGameplayStatics::GetAllActorsOfClass(GetWorld(), ALandingZone::StaticClass(), LandingZones);
	for (AActor* LZ_Actor : LandingZones)
	{
		ALandingZone* LZ = Cast<ALandingZone>(LZ_Actor);
     	double DistToLZ = FVector::Dist(LZ->GetExitTransform().GetLocation(), GetActorLocation());
		if (!IsValid(ClosestLandingZone))
		{
			ClosestLandingZone = LZ;
			DistToClosestLandingZone = DistToLZ;
			continue;
		}
		ClosestLandingZone = DistToClosestLandingZone >= DistToLZ ? LZ : ClosestLandingZone;
		DistToClosestLandingZone = DistToClosestLandingZone >= DistToLZ ? DistToLZ : DistToClosestLandingZone;
	}
	if (!IsValid(ClosestLandingZone)) return;
	GI->MainSaveGameInstanceRef->PlayerOpenWorldSpawnTransformSave = ClosestLandingZone->GetExitTransform();
	UGameplayStatics::SaveGameToSlot(GI->MainSaveGameInstanceRef, "MainSave", 0);
}
